@@ -12,8 +12,19 @@ from naclo.__model_schema import bleach_schema
 
 class Bleach:
     structure_types = sorted(['smiles', 'mol'])
+
+    assay_types = [
+        'vitro',
+        'vivo',
+        'percent'
+    ]
     
-    input_units = [
+    _nlm_assay_types = [
+        'vitro',
+        'vivo'
+    ]
+    
+    vitro_input_units = [
         'mixed',
         '-log(M)',
         'M',
@@ -24,7 +35,20 @@ class Bleach:
         'mg/L'
     ]
     
-    output_units = [  # No mixed
+    vivo_input_units = [
+        'mixed',
+        '-log(mol/kg)',
+        'mol/kg',
+        'mmol/kg',
+        'g/kg',
+        'mg/kg'
+    ]
+    
+    percent_input_units = [
+        '%'
+    ]
+    
+    vitro_output_units = [  # No mixed
         '-log(M)',
         'M',
         'mM',
@@ -34,6 +58,18 @@ class Bleach:
         'mg/L'
     ]
     
+    vivo_output_units = [
+        '-log(mol/kg)',
+        'mol/kg',
+        'mmol/kg',
+        'g/kg',
+        'mg/kg'
+    ]
+    
+    percent_output_units = [
+        '%'
+    ]
+    
     duplicate_methods = sorted(['average', 'remove', 'keep'])
     
     fragment_filters = sorted(['carbon_count', 'mw', 'atom_count', 'none'])
@@ -90,7 +126,8 @@ class Bleach:
 # -------------------------------------------------- PRIVATE METHODS ------------------------------------------------- #
     def _set_structure_cols(self) -> None:
         '''Sets Mol and SMILES columns using declared structure type. Uses defaults if not declared'''
-        self.mol_col = self.model['structure_col'] if self.model['structure_type'] == 'mol' else self._default_cols['mol']
+        self.mol_col = self.model['structure_col'] if self.model['structure_type'] == 'mol' else \
+            self._default_cols['mol']
         self.smiles_col = self.model['structure_col'] if self.model['structure_type'] == 'smiles' else \
             self._default_cols['smiles']
         self.inchi_key_col = self._default_cols['inchi_key']
@@ -194,9 +231,27 @@ class Bleach:
             
         return df
     
+    @staticmethod
+    def percent_average(df:pd.DataFrame, mol_col_name:str, inchi_key_col_name:str, value_col_name:str, input_units:str,
+                        units_col_name:Optional[str]) -> pd.DataFrame:
+        df = df.copy()
+        col_name = f'percent_avg_{value_col_name}'
+        df[col_name] = df[value_col_name]
+        
+        if input_units == 'mixed':
+            if units_col_name:
+                df = df.where(df[units_col_name] == '%').dropna()  # Remove any unit that isn't '%'
+            else:
+                raise ValueError('Must specify units_col_name if input_units is mixed')
+        elif input_units != '%':
+            raise ValueError(f'Input_units must be "%" or "mixed" if assay_type is "percent". Not {input_units}')
+        
+        return stse.duplicates.average(df, subsets=[inchi_key_col_name], average_by=col_name)  # NOTE: should save operation to log
+            
+    
     @staticmethod
     def nlm_geometric_mean(df:pd.DataFrame, mol_col_name:str, inchi_key_col_name:str, value_col_name:str,
-                           input_units:str, units_col_name:Optional[str]) -> pd.DataFrame:
+                           input_units:str, units_col_name:Optional[str], assay_type:str='vitro') -> pd.DataFrame:
         """Converts values in df to -log(M) and to computes the geometric mean
 
         Args:
@@ -210,21 +265,29 @@ class Bleach:
         Returns:
             pd.DataFrame: _description_
         """
-        df = df.copy()
-        col_name = f'neg_log_molar_{value_col_name}'  # This is necessary to avoid overwriting existing column
+        if assay_type not in Bleach._nlm_assay_types:
+            raise ValueError(f'units_type must be one of {Bleach._nlm_assay_types} for for negative log geometric mean')
         
-        # 1. Convert to -log(M)
+        df = df.copy()
+        
+        prefix = 'neg_log_molar' if assay_type == 'vitro' else 'neg_log_mol_per_kg'
+        col_name = f'{prefix}_{value_col_name}'  # This is necessary to avoid overwriting existing column
+        
+        # 1. Convert to -log(M) or -log(mol/kg)
         if input_units == 'mixed':  # Unit column
             if units_col_name:
-                # df[col_name] = UnitConverter(df[value_col_name], df[units_col_name], naclo.mol_weights(df[mol_col_name]))
-                df = UnitConverter.df_converter(df=df, mol_col_name=mol_col_name, value_col_name=value_col_name,
-                                                units_col_name=units_col_name, output_units='-log(m)', drop_na_units=False)
+                output_units = '-log(m)' if assay_type == 'vitro' else '-log(mol/kg)'
+                df, _ = UnitConverter.df_converter(df=df, mol_col_name=mol_col_name, value_col_name=value_col_name,
+                                                   units_col_name=units_col_name, output_units=output_units,
+                                                   drop_na_units=False)
             else:
                 raise ValueError('Must specify units_col_name if input_units is mixed')
         else:
             if units_col_name:
                 warnings.warn(bleach_warnings['CONFLICTING_INPUT_UNITS'], UserWarning)
-            df[col_name] = UnitConverter.convert(df[value_col_name].astype(float), input_units, '-log(m)')
+            out_unit = '-log(m)' if assay_type == 'vitro' else '-log(mol/kg)'
+            df[col_name] = UnitConverter.convert(df[value_col_name].astype(float), input_units, out_unit,
+                                                 mws=naclo.mol_weights(df[mol_col_name]), assay_type=assay_type)
         
         # 2. Average
         return stse.duplicates.average(df, subsets=[inchi_key_col_name], average_by=col_name)  # NOTE: should save operation to log
@@ -290,17 +353,25 @@ class Bleach:
     @staticmethod
     def handle_duplicates(df:pd.DataFrame, mol_col_name:str, inchi_key_col_name:str, target_col_name:Optional[str]=None,
                           input_units:Optional[str]=None, units_col_name:Optional[str]=None,
-                          method='average') -> pd.DataFrame:  # *
+                          method='average', assay_type='vitro') -> pd.DataFrame:  # *
         '''Computes inchi keys. Averages, removes, or keeps duplicates. ONLY BY INCHI KEY FOR NOW. Will return units in
         -log(M) if method is average, else original units'''
         avg_flag = False
         df = Bleach._append_inchi_keys(df, mol_col_name, inchi_key_col_name)
 
         if method == 'average' and target_col_name:
-            # Average only on -log(M)
-            df = Bleach.nlm_geometric_mean(df=df, mol_col_name=mol_col_name, inchi_key_col_name=inchi_key_col_name,
-                                           value_col_name=target_col_name, input_units=input_units,
-                                           units_col_name=units_col_name)  # NOTE: should save operation to log
+            # Average only on -log
+            if assay_type in Bleach._nlm_assay_types:
+                df = Bleach.nlm_geometric_mean(df=df, mol_col_name=mol_col_name, inchi_key_col_name=inchi_key_col_name,
+                                            value_col_name=target_col_name, input_units=input_units,
+                                            units_col_name=units_col_name,
+                                            assay_type=assay_type)  # NOTE: should save operation to log
+            elif assay_type == 'percent':
+                df = Bleach.percent_average(df=df, mol_col_name=mol_col_name, inchi_key_col_name=inchi_key_col_name,
+                                       value_col_name=target_col_name, input_units=input_units,
+                                       units_col_name=units_col_name)  # NOTE: should save operation to log
+            else:
+                raise ValueError(f'Invalid assay_type: {assay_type}. Must be one of {Bleach.assay_types}')
             avg_flag = True
         elif method == 'remove' or (method == 'average' and not target_col_name):
             if method == 'average':
@@ -313,29 +384,47 @@ class Bleach:
                                            target_col_name=self._target_col,
                                            input_units=self.model['input_units'],
                                            units_col_name=self.model['units_col'],
-                                           method=self.model['format_settings']['duplicate_compounds'])
+                                           method=self.model['format_settings']['duplicate_compounds'],
+                                           assay_type=self.model['assay_type'])
         
         # Set units to -log(m) if they were converted for geometric mean
         if self._average_flag:
-            self._target_col = f'neg_log_molar_{self._target_col}'  # Change column pointer
-            self._target_units = '-log(m)'
+            if self.model['assay_type'] == 'vitro':
+                self._target_col = f'neg_log_molar_{self._target_col}'  # Change column pointer
+                self._target_units = '-log(m)'
+            elif self.model['assay_type'] == 'vivo':
+                self._target_col = f'neg_log_mol_per_kg_{self._target_col}'
+                self._target_units = '-log(mol/kg)'
     
     @staticmethod  
-    def final_unit_conversion(df, input_units, output_units, units_col_name, mol_col_name, value_col_name, drop_na_units):
-        if input_units == 'mixed':
+    def final_unit_conversion(df, input_units, output_units, units_col_name, mol_col_name, value_col_name, drop_na_units,
+                              assay_type):
+        if output_units == '%':
+            return df, value_col_name  # Return orignal df and column name
+        elif input_units == 'mixed':
             if units_col_name:
                 return UnitConverter.df_converter(df=df, mol_col_name=mol_col_name, value_col_name=value_col_name,
-                                                units_col_name=units_col_name, output_units=output_units,
-                                                drop_na_units=drop_na_units)
+                                                  units_col_name=units_col_name, output_units=output_units,
+                                                  drop_na_units=drop_na_units)
             else:
                 raise ValueError('Must specify units_col_name if input_units is mixed')
         else:
             if units_col_name:
                 warnings.warn(bleach_warnings['CONFLICTING_INPUT_UNITS'], RuntimeWarning)
-                
-            units_label = 'neg_log_molar' if output_units.lower() == '-log(m)' else output_units  # -log(M) may screw up datasheet characters
-            df[f'{units_label}_{value_col_name}'] = UnitConverter.convert(df[value_col_name], input_units, output_units)
-            return df
+            
+            # '-log' may screw up datasheet characters
+            if output_units.lower() == '-log(m)':
+                units_label = 'neg_log_molar'
+            elif output_units.lower() == '-log(mol/kg)':
+                units_label = 'neg_log_mol_per_kg'
+            else:
+                units_label = output_units
+            
+            col_name = f'{units_label}_{value_col_name}'
+            df[col_name] = UnitConverter.convert(df[value_col_name], input_units, output_units,
+                                                                          mws=naclo.mol_weights(df[mol_col_name]),
+                                                                          assay_type=assay_type)
+            return df, col_name
             
     def _instance_final_unit_conversion(self):
         if self._target_units.lower() == self.model['output_units'].lower():
@@ -345,14 +434,23 @@ class Bleach:
             return
         
         # Has already been converted to -log(m)
-        if self._average_flag:
+        if self._average_flag and self.model['output_units'] != '%':
+            input_units = '-log(m)' if self.model['assay_type'] == 'vitro' else '-log(mol/kg)'
             col_name = '{0}_{1}'.format(self.model['output_units'], self.model['target_col'])
-            self._df[col_name] = UnitConverter.convert(self._df[self._target_col], '-log(m)', self.model['output_units'])
+            self._df[col_name] = UnitConverter.convert(self._df[self._target_col], input_units,
+                                                       self.model['output_units'],
+                                                       mws=naclo.mol_weights(self._df[self.mol_col]),
+                                                       assay_type=self.model['assay_type'])
         else:
-            self._df = Bleach.final_unit_conversion(df=self._df, input_units=self.model['input_units'],
-                                         output_units=self.model['output_units'],
-                                         units_col_name=self.model['units_col'], mol_col_name=self.mol_col,
-                                         value_col_name=self._target_col, drop_na_units=self.model['drop_na_units'])
+            self._df, self._target_col = Bleach.final_unit_conversion(df=self._df,
+                                                                      input_units=self.model['input_units'],
+                                                                      output_units=self.model['output_units'],
+                                                                      units_col_name=self.model['units_col'],
+                                                                      mol_col_name=self.mol_col,
+                                                                      value_col_name=self._target_col,
+                                                                      drop_na_units=self.model['drop_na_units'],
+                                                                      assay_type=self.model['assay_type'])
+        self._drop_na_targets()
 
     # Step 6
     @staticmethod
