@@ -24,3 +24,6 @@ with resources.open_text('naclo.assets', 'recognized_units.json') as f:
 
 with resources.open_text('naclo.assets', 'recognized_salts.json') as f:
     recognized_salts = json.load(f)
+    
+with resources.open_text('naclo.assets', 'bleach_warnings.yml') as f:
+    bleach_warnings = yaml.load(f, Loader=SafeLoader)
