@@ -8,9 +8,10 @@ bleach_schema = Schema({
 
     # Target data
     'target_col': Or(str, None),
-    'input_units': Or('mixed', '-log(M)', 'M', 'mM', 'µM', 'uM', 'nM', ''),
-    'output_units': Or('-log(M)', 'M', 'mM', 'µM', 'uM', 'nM', ''),
+    'input_units': Or('mixed', '-log(M)', '-log(mol/kg)', 'M', 'mM', 'µM', 'uM', 'nM', 'g/L', 'mg/L', 'mol/kg', 'mmol/kg', 'g/kg', 'mg/kg', '%', ''),
+    'output_units': Or('-log(M)', '-log(mol/kg)', 'M', 'mM', 'µM', 'uM', 'nM', 'g/L', 'mg/L', 'mol/kg', 'mmol/kg', 'g/kg', 'mg/kg', '%', ''),
     'units_col': str,
+    'assay_type': Or('vitro', 'vivo', 'percent'),
     'drop_na_units': bool,
 
     # Formatting
