@@ -588,6 +588,121 @@ class TestBleach(unittest.TestCase):
             list(bleach._df.columns)
         )
         
+    def test_assay_types(self):
+        test_units = ['M', 'mg/L', 'g/kg', '%', 'mg/kg', 'g/kg', 'uM']
+        test_targets = len(test_units) * [1]
+        
+        df = self.smiles_df
+        df['units'] = test_units
+        df['target'] = test_targets
+        model = self.default_model
+
+        model['structure_col'] = 'SMILES'
+        model['structure_type'] = 'smiles'
+        model['target_col'] = 'target'
+        model['input_units'] = 'mixed'
+        model['units_col'] = 'units'
+        model['format_settings']['remove_na_targets'] = True
+        
+        # Test vitro
+        model['assay_type'] = 'vitro'
+        model['output_units'] = 'M'
+        
+        bleach = Bleach(df, model)
+        bleach.main()
+        
+        expected_smiles = df.iloc[:2]['SMILES'].tolist()
+        expected_targets = [1, 1e-3 / naclo.mol_weights(naclo.smiles_2_mols([df.iloc[1]['SMILES']]))[0]]
+
+        self.assertEqual(
+            bleach._df['SMILES'].tolist(),
+            expected_smiles
+        )
+        self.assertTrue(
+            np.allclose(
+                bleach._df['M_target'].to_numpy(),
+                expected_targets
+            )
+        )
+
+        # Test vivo
+        model['assay_type'] = 'vivo'
+        model['output_units'] = 'mol/kg'
+        
+        bleach = Bleach(df, model)
+        bleach.main()
+        
+        expected_smiles = [x.split('.')[0] for x in df.iloc[3:5]['SMILES'].tolist()]
+        expected_targets = np.array([1 / naclo.mol_weights(naclo.smiles_2_mols([x]))[0] for x in expected_smiles])
+        expected_targets *= [1, 1e-3]  # Multiply prefixes
+        
+        self.assertEqual(
+            bleach._df['SMILES'].tolist(),
+            expected_smiles
+        )
+        self.assertTrue(
+            np.allclose(
+                bleach._df['mol/kg_target'].to_numpy(),
+                expected_targets
+            )
+        )
+        
+        # Test percent
+        model['assay_type'] = 'percent'
+        model['output_units'] = '%'
+        
+        bleach = Bleach(df, model)
+        bleach.main()
+        
+        expected_smiles = [x.split('.')[0] for x in [df.iloc[3]['SMILES']]]
+        expected_targets = test_targets[3]  # No conversion is done
+        
+        self.assertEqual(
+            bleach._df['SMILES'].tolist(),
+            expected_smiles
+        )
+        self.assertTrue(
+            np.allclose(
+                bleach._df['percent_avg_target'].to_numpy(),
+                expected_targets
+            )
+        )
+        
+    def test_neg_log_mol_conversion(self):
+        df = self.smiles_df
+        df['target'] = len(df) * [1]
+        
+        model = self.default_model
+        model['structure_col'] = 'SMILES'
+        model['structure_type'] = 'smiles'
+        model['target_col'] = 'target'
+        model['input_units'] = '-log(M)'
+        model['output_units'] = '-log(M)'
+        
+        bleach = naclo.Bleach(df, model)
+        bleach.main()
+        
+        # Ensure no conversion is done
+        self.assertEqual(
+            bleach._df['neg_log_molar_target'].tolist(),
+            len(bleach._df) * [1]
+        )
+        
+        # Convert from -log(M) to uM
+        model['output_units'] = 'uM'
+        
+        bleach = naclo.Bleach(df, model)
+        bleach.main()
+        
+        inv_neg_log = lambda x: 10**(-1*x)
+        
+        self.assertTrue(
+            np.allclose(
+                bleach._df['uM_target'].tolist(),
+                [(1e6)*inv_neg_log(x) for x in bleach._df['neg_log_molar_target'].tolist()]
+            )
+        )
+    
     
 if __name__ == '__main__':
     unittest.main()
